#! /usr/bin/env python3
import logging
import subprocess
import time

import psutil
import requests as requests


class NetworkOperations:
    @staticmethod
    def check_internet_connection():
        """Detect an internet connection."""

        connection = None
        # noinspection PyBroadException
        try:
            r = requests.get("https://google.com")
            r.raise_for_status()
            logging.debug("Internet connection detected.")
            connection = True
        except Exception as ex:
            logging.debug("Internet connection not detected.")
            logging.debug(f"{ex}")
            connection = False
        finally:
            return connection

    def get_signal_strength(self="wlan0"):
        """Get the signal strength of the wireless network."""
        try:
            output = subprocess.check_output(
                ["/usr/sbin/iwconfig", self], stderr=subprocess.STDOUT, universal_newlines=True
            )
            lines = output.split("\n")

            for line in lines:
                if "Signal level" in line:
                    index_start = line.index("Signal level=") + len("Signal level=")
                    index_end = line.index(" dBm")
                    signal_level = line[index_start:index_end]
                    return int(signal_level)

            # If signal strength information is not found
            return None
        except subprocess.CalledProcessError as e:
            logging.debug(f"Error running iwconfig: {e.output}")
            return None

    @staticmethod
    def get_ip_address():
        """Get the IP address of the wireless network."""
        return psutil.net_if_addrs()["wlan0"][0].address

    @staticmethod
    def get_current_traffic():
        """Get the current network traffic."""

        interval = 1
        # Getting network traffic information
        net_io_start = psutil.net_io_counters()

        # Waiting for the specified interval
        time.sleep(interval)

        # Getting network traffic information after the start values
        net_io_end = psutil.net_io_counters()

        # Calculating the difference in network traffic
        download_bytes = net_io_end.bytes_recv - net_io_start.bytes_recv
        upload_bytes = net_io_end.bytes_sent - net_io_start.bytes_sent

        # Converting bytes to megabytes and calculating the speed in MB/s
        download_speed = download_bytes / interval / (1024 * 1024)
        upload_speed = upload_bytes / interval / (1024 * 1024)

        return download_speed, upload_speed



# /*****************************************************************************
# * | File        :	  epdconfig.py
# * | Author      :   Waveshare team
# * | Function    :   Hardware underlying interface
# * | Info        :
# *----------------
# * | This version:   V1.2
# * | Date        :   2022-10-29
# * | I nfo        :
# ******************************************************************************
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#

import logging
import os
import subprocess
import sys
import time

logger = logging.getLogger(__name__)


class RaspberryPi:
    # Pin definition
    RST_PIN = 17
    DC_PIN = 25
    CS_PIN = 8
    BUSY_PIN = 24
    PWR_PIN = 18

    def __init__(self):
        import gpiozero
        import spidev

        self.SPI = spidev.SpiDev()
        self.GPIO_RST_PIN = gpiozero.LED(self.RST_PIN)
        self.GPIO_DC_PIN = gpiozero.LED(self.DC_PIN)
        # self.GPIO_CS_PIN     = gpiozero.LED(self.CS_PIN)
        self.GPIO_PWR_PIN = gpiozero.LED(self.PWR_PIN)
        self.GPIO_BUSY_PIN = gpiozero.Button(self.BUSY_PIN, pull_up=False)

    def digital_write(self, pin, value):
        if pin == self.RST_PIN:
            if value:
                self.GPIO_RST_PIN.on()
            else:
                self.GPIO_RST_PIN.off()
        elif pin == self.DC_PIN:
            if value:
                self.GPIO_DC_PIN.on()
            else:
                self.GPIO_DC_PIN.off()
        # elif pin == self.CS_PIN:
        #     if value:
        #         self.GPIO_CS_PIN.on()
        #     else:
        #         self.GPIO_CS_PIN.off()
        elif pin == self.PWR_PIN:
            if value:
                self.GPIO_PWR_PIN.on()
            else:
                self.GPIO_PWR_PIN.off()

    def digital_read(self, pin):
        if pin == self.BUSY_PIN:
            return self.GPIO_BUSY_PIN.value
        elif pin == self.RST_PIN:
            return self.RST_PIN.value
        elif pin == self.DC_PIN:
            return self.DC_PIN.value
        # elif pin == self.CS_PIN:
        #     return self.CS_PIN.value
        elif pin == self.PWR_PIN:
            return self.PWR_PIN.value

    def delay_ms(self, delaytime):
        time.sleep(delaytime / 1000.0)

    def spi_writebyte(self, data):
        self.SPI.writebytes(data)

    def spi_writebyte2(self, data):
        self.SPI.writebytes2(data)

    def module_init(self):
        self.GPIO_PWR_PIN.on()

        # SPI device, bus = 0, device = 0
        self.SPI.open(0, 0)
        self.SPI.max_speed_hz = 4000000
        self.SPI.mode = 0b00
        return 0

    def module_exit(self, cleanup=False):
        logger.debug("spi end")
        self.SPI.close()

        self.GPIO_RST_PIN.off()
        self.GPIO_DC_PIN.off()
        self.GPIO_PWR_PIN.off()
        logger.debug("close 5V, Module enters 0 power consumption ...")

        if cleanup:
            self.GPIO_RST_PIN.close()
            self.GPIO_DC_PIN.close()
            # self.GPIO_CS_PIN.close()
            self.GPIO_PWR_PIN.close()
            self.GPIO_BUSY_PIN.close()


class JetsonNano:
    # Pin definition
    RST_PIN = 17
    DC_PIN = 25
    CS_PIN = 8
    BUSY_PIN = 24
    PWR_PIN = 18

    def __init__(self):
        import ctypes

        find_dirs = [
            os.path.dirname(os.path.realpath(__file__)),
            "/usr/local/lib",
            "/usr/lib",
        ]
        self.SPI = None
        for find_dir in find_dirs:
            so_filename = os.path.join(find_dir, "sysfs_software_spi.so")
            if os.path.exists(so_filename):
                self.SPI = ctypes.cdll.LoadLibrary(so_filename)
                break
        if self.SPI is None:
            raise RuntimeError("Cannot find sysfs_software_spi.so")

        import Jetson.GPIO

        self.GPIO = Jetson.GPIO

    def digital_write(self, pin, value):
        self.GPIO.output(pin, value)

    def digital_read(self, pin):
        return self.GPIO.input(self.BUSY_PIN)

    def delay_ms(self, delaytime):
        time.sleep(delaytime / 1000.0)

    def spi_writebyte(self, data):
        self.SPI.SYSFS_software_spi_transfer(data[0])

    def spi_writebyte2(self, data):
        for i in range(len(data)):
            self.SPI.SYSFS_software_spi_transfer(data[i])

    def module_init(self):
        self.GPIO.setmode(self.GPIO.BCM)
        self.GPIO.setwarnings(False)
        self.GPIO.setup(self.RST_PIN, self.GPIO.OUT)
        self.GPIO.setup(self.DC_PIN, self.GPIO.OUT)
        self.GPIO.setup(self.CS_PIN, self.GPIO.OUT)
        self.GPIO.setup(self.PWR_PIN, self.GPIO.OUT)
        self.GPIO.setup(self.BUSY_PIN, self.GPIO.IN)

        self.GPIO.output(self.PWR_PIN, 1)

        self.SPI.SYSFS_software_spi_begin()
        return 0

    def module_exit(self):
        logger.debug("spi end")
        self.SPI.SYSFS_software_spi_end()

        logger.debug("close 5V, Module enters 0 power consumption ...")
        self.GPIO.output(self.RST_PIN, 0)
        self.GPIO.output(self.DC_PIN, 0)
        self.GPIO.output(self.PWR_PIN, 0)

        self.GPIO.cleanup([self.RST_PIN, self.DC_PIN, self.CS_PIN, self.BUSY_PIN, self.PWR_PIN])


class SunriseX3:
    # Pin definition
    RST_PIN = 17
    DC_PIN = 25
    CS_PIN = 8
    BUSY_PIN = 24
    PWR_PIN = 18
    Flag = 0

    def __init__(self):
        import Hobot.GPIO
        import spidev

        self.GPIO = Hobot.GPIO
        self.SPI = spidev.SpiDev()

    def digital_write(self, pin, value):
        self.GPIO.output(pin, value)

    def digital_read(self, pin):
        return self.GPIO.input(pin)

    def delay_ms(self, delaytime):
        time.sleep(delaytime / 1000.0)

    def spi_writebyte(self, data):
        self.SPI.writebytes(data)

    def spi_writebyte2(self, data):
        # for i in range(len(data)):
        #     self.SPI.writebytes([data[i]])
        self.SPI.xfer3(data)

    def module_init(self):
        if self.Flag == 0:
            self.Flag = 1
            self.GPIO.setmode(self.GPIO.BCM)
            self.GPIO.setwarnings(False)
            self.GPIO.setup(self.RST_PIN, self.GPIO.OUT)
            self.GPIO.setup(self.DC_PIN, self.GPIO.OUT)
            self.GPIO.setup(self.CS_PIN, self.GPIO.OUT)
            self.GPIO.setup(self.PWR_PIN, self.GPIO.OUT)
            self.GPIO.setup(self.BUSY_PIN, self.GPIO.IN)

            self.GPIO.output(self.PWR_PIN, 1)

            # SPI device, bus = 0, device = 0
            self.SPI.open(2, 0)
            self.SPI.max_speed_hz = 4000000
            self.SPI.mode = 0b00
            return 0
        else:
            return 0

    def module_exit(self):
        logger.debug("spi end")
        self.SPI.close()

        logger.debug("close 5V, Module enters 0 power consumption ...")
        self.Flag = 0
        self.GPIO.output(self.RST_PIN, 0)
        self.GPIO.output(self.DC_PIN, 0)
        self.GPIO.output(self.PWR_PIN, 0)

        self.GPIO.cleanup([self.RST_PIN, self.DC_PIN, self.CS_PIN, self.BUSY_PIN], self.PWR_PIN)


process = subprocess.Popen(
    "cat /proc/cpuinfo | grep Raspberry", shell=True, stdout=subprocess.PIPE, text=True
)
output, _ = process.communicate()

if "Raspberry" in output:
    implementation = RaspberryPi()
elif os.path.exists("/sys/bus/platform/drivers/gpio-x3"):
    implementation = SunriseX3()
else:
    implementation = JetsonNano()

for func in [x for x in dir(implementation) if not x.startswith("_")]:
    setattr(sys.modules[__name__], func, getattr(implementation, func))

### END OF FILE ###



#! /usr/bin/env python3

import logging
import signal
import sys
import time
import os

import schedule

from zlsnasdisplay.display_renderer import DisplayRenderer

# GET ENVIRONMENT VARIABLES
LOG_LEVEL = os.getenv("LOG_LEVEL", "INFO").upper()
SENTRY_DSN = os.getenv("SENTRY_DSN", False)
DISPLAY_IMAGE_PATH = os.getenv("DISPLAY_IMAGE_PATH", False)

# Configure logging level
logging.basicConfig(level=LOG_LEVEL)

if SENTRY_DSN:
    import sentry_sdk

    # Initialize Sentry for error tracking
    sentry_sdk.init(SENTRY_DSN)

# Detect sudo
IS_ROOT = os.getuid() == 0

if not IS_ROOT:
    logging.warning("The script does not run as root. Cannot perform apt update!")

display_renderer = DisplayRenderer(DISPLAY_IMAGE_PATH, IS_ROOT)


# Define signal_handler function to catch SIGINT (Ctrl+C)
def signal_handler(sig, frame):
    """Signal handler function to catch SIGINT (Ctrl+C) and exit the program."""
    logging.info("Exiting the program...")
    # Clear all scheduled jobs
    schedule.clear()
    time.sleep(1)
    # Wait for running jobs to complete
    while schedule.jobs:
        time.sleep(1)

    display_renderer.go_to_sleep()

    sys.exit(0)


def main():
    """Main function to run the program."""

    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGHUP, signal_handler)

    # Render CPU load every 30 seconds
    schedule.every(30).seconds.do(display_renderer.render_cpu_load)
    # Get updates every 3 hours
    schedule.every(3).hours.do(display_renderer.get_updates)
    # Render signal strength every minute
    schedule.every(1).minutes.do(display_renderer.render_signal_strength)
    # Render memory usage every minute
    schedule.every(1).minutes.do(display_renderer.render_mem)
    # Render NVMe stats every minute
    schedule.every(1).minutes.do(display_renderer.render_nvme_stats)
    # Render fan speed every 10 seconds
    schedule.every(10).seconds.do(display_renderer.render_fan_speed)
    # Render IP address every hour
    schedule.every(1).hours.do(display_renderer.render_ip_address)
    # Render uptime every minute
    schedule.every(1).minutes.do(display_renderer.render_uptime)
    # Render current traffic every 10 seconds
    schedule.every(10).seconds.do(display_renderer.render_current_traffic)
    # Render current traffic every 30 seconds
    schedule.every(30).seconds.do(display_renderer.check_net)

    schedule.run_all()

    while True:
        """ Run the scheduled tasks."""
        schedule.run_pending()
        time.sleep(1)


if __name__ == "__main__":
    main()



#! /usr/bin/env python3

import os

from PIL import Image, ImageDraw, ImageFont

from zlsnasdisplay.display_controller import DisplayController
from zlsnasdisplay.network_operations import NetworkOperations
from zlsnasdisplay.system_operations import SystemOperations


class DisplayRenderer:
    def __init__(self, display_image_path, is_root):
        """Initialize the display renderer"""
        self.display_controller = DisplayController()

        self.display_controller.clear_display()

        self.display_image_path = display_image_path
        self.is_root = is_root

        # Define directories for fonts and the e-paper display library
        fontdir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "fonts")

        # Load fonts
        self.font34 = ImageFont.truetype(os.path.join(fontdir, "Ubuntu-Regular.ttf"), 34)
        self.font24 = ImageFont.truetype(os.path.join(fontdir, "Ubuntu-Regular.ttf"), 24)
        self.font14 = ImageFont.truetype(os.path.join(fontdir, "Ubuntu-Light.ttf"), 14)

        self.nfont50 = ImageFont.truetype(os.path.join(fontdir, "MaterialSymbolsRounded.ttf"), 50)
        self.nfont24 = ImageFont.truetype(os.path.join(fontdir, "MaterialSymbolsRounded.ttf"), 24)
        self.nfont14 = ImageFont.truetype(os.path.join(fontdir, "MaterialSymbolsRounded.ttf"), 14)

        # Create an image
        self.image = Image.new(
            "1", (self.display_controller.epd.height, self.display_controller.epd.width), 255
        )
        self.draw = ImageDraw.Draw(self.image)

        # Draw a horizontal line
        self.draw.line([(98, 10), (98, 110)], fill=0, width=0)
        # Draw a horizontal line
        self.draw.line([(203, 10), (203, 110)], fill=0, width=0)
        # Draw a horizontal line
        self.draw.line([(0, 110), (297, 110)], fill=0, width=0)

    def update_display_and_save_image(self):
        """Update the display and save the image"""
        self.display_controller.update_display(self.image)
        if self.display_image_path:
            self.image.save(self.display_image_path, "PNG")

    def render_cpu_load(self):
        """Render CPU load"""
        self.draw.rectangle((0, 0, 97, 67), fill=255)

        self.draw.line(((26, 10), (97, 10)), fill=0, width=0)

        self.draw.text((1, 0), "cpu", font=self.font14, fill=0)
        # Draw CPU usage
        self.draw.text((10, 12), "\ue30d", font=self.nfont24, fill=0)  # Unicode icon for CPU
        self.draw.text(
            (40, 12), f"{SystemOperations().get_cpu_load()}%", font=self.font24, fill=0
        )  # CPU percentage
        self.draw.text(
            (10, 42), "\ue1ff", font=self.nfont24, fill=0
        )  # Unicode icon for temperature
        self.draw.text(
            (40, 42), f"{SystemOperations().get_cpu_temperature()}°C", font=self.font24, fill=0
        )  # CPU temperature

        self.update_display_and_save_image()

    def get_updates(self):
        """Get updates for the display"""
        self.draw.rectangle((204, 68, 248, 100), fill=255)

        self.draw.text((204, 67), "apt", font=self.font14, fill=0)
        self.draw.line([(226, 76), (248, 76)], fill=0, width=0)
        number_of_updates = SystemOperations().check_updates(self.is_root)
        if number_of_updates == 0:
            self.draw.text((214, 80), "\ue8e8", font=self.nfont24, fill=0)
        else:
            self.draw.text(
                (214, 80), f"{number_of_updates}", font=self.font24, fill=0
            )  # Number of available updates

        self.update_display_and_save_image()

    def check_net(self):
        """Check network status"""
        self.draw.rectangle((249, 79, 296, 109), fill=255)

        self.draw.text((250, 67), "net", font=self.font14, fill=0)
        self.draw.line([(249, 76), (249, 110)], fill=0, width=0)
        self.draw.line([(272, 76), (297, 76)], fill=0, width=0)
        if NetworkOperations.check_internet_connection():
            self.draw.text((260, 80), "\ue2bf", font=self.nfont24, fill=0)
        else:
            self.draw.text((260, 80), "\uf1ca", font=self.nfont24, fill=0)

        self.update_display_and_save_image()

    def render_signal_strength(self):
        """Render signal strength"""
        self.draw.rectangle((125, 111, 204, 128), fill=255)

        signal = NetworkOperations.get_signal_strength()

        if signal is not None:
            self.draw.text((125, 110), "\ue63e", font=self.nfont14, fill=0)  # Unicode icon for wifi
            self.draw.text((140, 110), f"{signal} dBm", font=self.font14, fill=0)  # CPU temperature
        else:
            self.draw.text((125, 110), "\ue1da", font=self.nfont14, fill=0)  # Unicode icon for wifi

        self.update_display_and_save_image()

    def render_mem(self):
        """Render memory stats"""
        self.draw.rectangle((0, 73, 97, 109), fill=255)

        self.draw.text((1, 67), "mem", font=self.font14, fill=0)
        self.draw.line([(34, 76), (97, 76)], fill=0, width=0)
        self.draw.text((10, 80), "\ue322", font=self.nfont24, fill=0)  # Unicode icon for memory
        self.draw.text(
            (40, 80), f"{SystemOperations.get_mem()}%", font=self.font24, fill=0
        )  # Memory percentage

        self.update_display_and_save_image()

    def render_nvme_stats(self):
        """Render NVME stats"""
        self.draw.rectangle((111, 0, 202, 68), fill=255)

        # DISK
        self.draw.text((100, 0), "nvme", font=self.font14, fill=0)
        self.draw.line([(138, 10), (203, 10)], fill=0, width=0)
        self.draw.text(
            (108, 12), "\uf7a4", font=self.nfont24, fill=0
        )  # Unicode icon for Hard Drive
        self.draw.text(
            (138, 12), f"{SystemOperations.get_nvme_usage()}%", font=self.font24, fill=0
        )  # CPU percentage
        self.draw.text(
            (108, 42), "\ue1ff", font=self.nfont24, fill=0
        )  # Unicode icon for temperature
        self.draw.text(
            (138, 42), f"{SystemOperations.get_nvme_temp()}°C", font=self.font24, fill=0
        )  # Nvme temperature

        self.update_display_and_save_image()

    def render_fan_speed(self):
        """Render fan speed"""
        self.draw.rectangle((100, 70, 202, 109), fill=255)

        self.draw.text((100, 67), "fan", font=self.font14, fill=0)
        self.draw.line([(122, 76), (203, 76)], fill=0, width=0)
        self.draw.text((108, 80), "\uf168", font=self.nfont24, fill=0)  # Unicode icon for fan
        self.draw.text(
            (138, 80), f"{SystemOperations.get_fan_speed()}", font=self.font24, fill=0
        )  # Fan speed

        self.update_display_and_save_image()

    def render_ip_address(self):
        """Render IP address"""
        self.draw.rectangle((0, 111, 109, 128), fill=255)

        self.draw.text((5, 110), "\ue80d", font=self.nfont14, fill=0)  # Unicode icon for network
        self.draw.text(
            (20, 110), f"{NetworkOperations.get_ip_address()}", font=self.font14, fill=0
        )  # Ip address

        self.update_display_and_save_image()

    def render_uptime(self):
        """Render uptime"""
        self.draw.rectangle((205, 111, 296, 128), fill=255)

        uptime = SystemOperations.get_uptime()

        self.draw.text((205, 110), "\ue923", font=self.nfont14, fill=0)  # Unicode icon for uptime
        self.draw.text(
            (220, 110), f"{uptime[0]}d {uptime[1]}h {uptime[2]}m", font=self.font14, fill=0
        )  # uptime

        self.update_display_and_save_image()

    def render_current_traffic(self):
        """Render current traffic"""
        self.draw.rectangle((204, 0, 296, 68), fill=255)

        network = NetworkOperations.get_current_traffic()
        self.draw.text((204, 0), "net (MB/s)", font=self.font14, fill=0)
        self.draw.line([(270, 10), (296, 10)], fill=0, width=0)
        self.draw.text((208, 12), "\uf090", font=self.nfont24, fill=0)  # Unicode icon download
        self.draw.text((233, 12), f"{round(network[0], 2)}", font=self.font24, fill=0)  # download
        self.draw.text((208, 42), "\uf09b", font=self.nfont24, fill=0)  # Unicode icon for upload
        self.draw.text((233, 42), f"{round(network[1], 2)}", font=self.font24, fill=0)  # upload

        self.update_display_and_save_image()

    def go_to_sleep(self):
        """Render the display"""
        self.draw.rectangle((0, 0, 296, 128), fill=0)
        self.draw.line([(140, 15), (140, 110)], fill=255, width=0)
        self.draw.text((155, 36), "ZlsNas", font=self.font34, fill=255)
        self.draw.text((70, 36), "\ue80d", font=self.nfont50, fill=255)
        self.draw.text((160, 71), "Sleeping...", font=self.font14, fill=255)

        self.update_display_and_save_image()

        self.display_controller.sleep_display()
